unit CryptoAPIRNG;

interface

uses SysUtils;

const
  C_Crypto_Seed_Length = 80;

type
  ECryptoAPI = class(Exception);
  TSeedBuff = array [0..C_Crypto_Seed_Length - 1] of byte;

Procedure GetCryptoSeed(var aSeedBuff: TSeedBuff);
function GetCryptoSeedString: string;

implementation

uses Windows, Registry;

const
  C_Crpyto_RNG_Key = '\SOFTWARE\Microsoft\Cryptography\RNG';
  C_Crypto_Seed_Value = 'Seed';

function GetCryptoSeedString: string;
var
  seedByte: Byte;
  seedIdx: Integer;
  seedBuf: TSeedBuff;
begin
  GetCryptoSeed(SeedBuf);
  for seedIdx := 0 to C_Crypto_Seed_Length - 1 do
  begin
    seedByte := seedBuf[seedIdx];
    Result := Result + IntToHex(seedByte, 2);
  end;
end;

Procedure GetCryptoSeed(var aSeedBuff: TSeedBuff);
var
  seedLen: Integer;
  reg: TRegistry;
begin
  reg := TRegistry.Create;
  try
    reg.RootKey := HKEY_LOCAL_MACHINE;
    if reg.KeyExists(C_Crpyto_RNG_Key) and
       reg.OpenKeyReadOnly(C_Crpyto_RNG_Key) and
       reg.ValueExists(C_Crypto_Seed_Value) then
    begin
      seedLen := reg.GetDataSize(C_Crypto_Seed_Value); // Get the reported size
      Assert(seedLen = C_Crypto_Seed_Length,'Seed is not expected length!');
      Assert(reg.ReadBinaryData(C_Crypto_Seed_Value, aSeedBuff, SeedLen) = seedLen,
        'A different length of bytes were read then were reported.');
    end
    else
    begin
      raise ECryptoAPI.Create('Either unable to find or open CryptoAPI RNG seed value in registry.');
    end;
  finally
    reg.Free;
  end;
end;

end.
