unit CryptoHash;

interface

uses Windows, SysUtils, wcrypt2, CryptoWrapper, CryptoProvider;

type
  TCryptoHashType = (chtMD2, chtMD4, chtMD5, chtSHA, chtSHA1, chtMAC);
  TCryptoHash = class(TCryptoWrapper)
  private
    fProvider: TCryptoProvider;
    fhHash: HCRYPTHASH;
    fHashType: TCryptoHashType;
  public
    constructor create(const aProvider: TCryptoProvider;
      const aHashType: TCryptoHashType);
    function HashString(const aPlainText: string):string;
    function HexString(const aPlainText: string): string;
    destructor Destroy; override;
  end;


implementation

uses
  CryptoConsts;


{ TCryptoHash }

constructor TCryptoHash.create(const aProvider: TCryptoProvider;
  const aHashType: TCryptoHashType);
begin
  fProvider := aProvider;
  fHashType := aHashType;

  if not CryptCreateHash( fProvider.GetProviderHandle ,
    C_CryptoHashTypes[ fHashType ], 0, 0, @fhHash) then
    ECryptoException.RaiseLastCryptoError;
end;

destructor TCryptoHash.Destroy;
begin
  CryptDestroyHash(fhHash);
  inherited;
end;

function TCryptoHash.HashString(const aPlainText: string): string;
var
  lHash: array [0..$7F] of Char;
  lHashLen: dWord;
  I: Integer;
begin
  Result := '';
  if CryptHashData(fhHash, @aPlainText[1], length(aPlainText), 0) then
  begin
    if CryptGetHashParam(fhHash, HP_HASHVAL, @lHash[0], @lHashLen, 0) then
    begin
      for I := 0 to lHashLen - 1 do
        Result := Result + lHash[i];
    end else ECryptoException.RaiseLastCryptoError;
  end else ECryptoException.RaiseLastCryptoError;
end;

function TCryptoHash.HexString(const aPlainText: string): string;
var
  lHash: array [0..$7F] of byte;
  lHashLen: dWord;
  I: Integer;
begin
  Result := '';
  if CryptHashData(fhHash, @aPlainText[1], length(aPlainText), 0) then
  begin
    if CryptGetHashParam(fhHash, HP_HASHVAL, @lHash[0], @lHashLen, 0) then
    begin
      for I := 0 to lHashLen - 1 do
        Result := Result + IntToHex(lHash[i],2);
    end else ECryptoException.RaiseLastCryptoError;
  end else ECryptoException.RaiseLastCryptoError;
end;


end.
