unit CryptoProvider;

interface

uses Windows, SysUtils, wcrypt2, CryptoWrapper;

type
  TCryptoProverType =
    ( cptRSAFull, cptRSASig, cptDSS, cptFortezza, cptExchange, cptSSL );
  TCryptoContext =
    ( ccVerify, ccNewKeySet, ccDelKeySet, ccMachKeySet );
  TCryptoContextSet = set of TCryptoContext;

const
  ccVerifyMachKeySet = [ccVerify, ccMachKeySet];

type
  // Cryptographic provider context
  TCryptoProvider = class(TCryptoWrapper)
  private
    fhCryptProv: HCRYPTPROV;
    fCryptoContextSet: TCryptoContextSet;
    fProviderType: TCryptoProverType;
  protected
  public
    constructor Create(const aProviderType: TCryptoProverType;
      const aContext: TCryptoContextSet );
    destructor Destroy; override;
    class function ORConextSet(const aContextSet: TCryptoContextSet): DWORD;
    function GetProviderHandle: HCRYPTPROV;
    property Context: TCryptoContextSet read fCryptoContextSet;
    property ProviderType: TCryptoProverType read fProviderType;
    function RandomData(const aLen: Integer):string;
  end;

implementation

uses
  CryptoConsts;

{ TCryptoContainer }

constructor TCryptoProvider.Create( const aProviderType: TCryptoProverType;
                                    const aContext: TCryptoContextSet);
var
  lContextSet: DWord;
begin
  fCryptoContextSet := aContext;
  fProviderType := aProviderType;
  lContextSet := ORConextSet(fCryptoContextSet);

  if not CryptAcquireContext(
      @fhCryptProv,
      nil,
      nil,
      C_CryptoProviderTypes[ fProviderType ],
      lContextSet ) then
  ECryptoException.RaiseLastCryptoError;
end;

destructor TCryptoProvider.Destroy;
begin
  CryptReleaseContext(fhCryptProv, 0);
  inherited;
end;

function TCryptoProvider.GetProviderHandle: HCRYPTPROV;
begin
  Result := fhCryptProv;
end;

class function TCryptoProvider.ORConextSet(
  const aContextSet: TCryptoContextSet): DWORD;
var
  lContext: TCryptoContext;
begin
  Result := 0;
  for lContext := ccVerify to ccMachKeySet do
  begin
    if lContext in aContextSet then
      Result := Result or C_CryptoContexts[ lContext ];
  end;

end;

function TCryptoProvider.RandomData(const aLen: Integer): string;
const
  C_BlockSize = 1024;
var
  lBlock : array [0..C_BlockSize - 1] of byte;
  lToGet, lPos: Integer;
begin
  lPos := 0;
  Result := '';
  while lPos < aLen do
  begin
    lToGet := aLen - lPos;
    if lToGet > C_BlockSize then
    begin
      lToGet := C_BlockSize;
    end;

    CryptGenRandom( fhCryptProv, lToGet, @lBlock[0]);
    Result := Result + EncodeBuffer(lBlock,lToGet);
    inc(lPos, lToGet);
  end;
end;

end.

