unit CryptoWrapper;

interface

uses
  Windows, SysUtils, wcrypt2;

type
  TCryptoWrapper = class abstract
  public
    class function EncodeBuffer(const lBuf; const lBufLen: Cardinal): string; static;
    class function Decode2Buffer(const aEncStr: string; var lBuf): Cardinal; static;
    class function String2Buffer(const aStr: string; var lBuf): Cardinal; static;
  end;

  ECryptoException = class(Exception)
  private
    FErrorValue: DWord;
    FDescription: string;
  public
    property Description: string read FDescription;
    property ErrorValue: DWord read FErrorValue;
    constructor Create(const aCryptoErrorValue: DWord);
    class procedure RaiseLastCryptoError;
    class procedure RaiseCryptoError(const aErrorCode: Cardinal);
  end;


implementation

uses
  Classes, EncdDecd, Wcrypt2Errors;

{ TCryptoWrapper }

class function TCryptoWrapper.EncodeBuffer(const lBuf; const lBufLen: Cardinal): string;
var
  lSource: TMemoryStream;
  lDest: TStringStream;
begin
  lSource := TMemoryStream.Create;
  try
    lSource.Write(lBuf, lBufLen);
    lSource.Position := 0;
    lDest := TStringStream.Create('');
    try
      EncodeStream(lSource, lDest);
      Result := lDest.DataString;
    finally
      lDest.Free;
    end;
  finally
    lSource.Free;
  end;
end;

class function TCryptoWrapper.String2Buffer(const aStr: string;
  var lBuf): Cardinal;
var
  lSource: TStringStream;
begin
  lSource := TStringStream.Create( aStr );
  try
    lSource.Read(lBuf, lSource.Size);
  finally
    lSource.Free;
  end;
end;

class function TCryptoWrapper.Decode2Buffer(const aEncStr: string; var lBuf): Cardinal;
var
  lSource: TStringStream;
  lDest: TMemoryStream;
begin
  lSource := TStringStream.Create( aEncStr );
  try
    lDest := TMemoryStream.Create;
    try
      DecodeStream( lSource, lDest );
      lDest.Position := 0;  // Always set the position after operating on the stream
      lDest.Read(lBuf, lSource.Size);
    finally
      lDest.Free;
    end;

  finally
    lSource.Free;
  end;
end;

{ ECryptoException }

constructor ECryptoException.Create(const aCryptoErrorValue: DWord);
begin
  fErrorValue := aCryptoErrorValue;
  FDescription := GetCryptErrorDescription( aCryptoErrorValue );
  Message := GetCryptErrorMessage( aCryptoErrorValue );
end;

class procedure ECryptoException.RaiseCryptoError(const aErrorCode: Cardinal);
  function ReturnAddr: Pointer;
  asm
          MOV     EAX,[EBP + 4]
  end;
begin
  raise ECryptoException.Create( aErrorCode ) at ReturnAddr;
end;

class procedure ECryptoException.RaiseLastCryptoError;
  function ReturnAddr: Pointer;
  asm
          MOV     EAX,[EBP + 4]
  end;
begin
  raise ECryptoException.Create( GetLastError ) at ReturnAddr;
end;

end.
