unit Wcrypt2DemoUnit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls,

  Wcrypt2, CryptoConsts, CryptoHash, CryptoProvider, CryptoWrapper,
  CryptoKey, Wcrypt2Errors, NoiseSpunge, Isaac;

type
  TForm1 = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    Memo1: TMemo;
    btnHash: TButton;
    Edit1: TEdit;
    txtPlainText: TMemo;
    btnKeyGen: TButton;
    btnEncrypt: TButton;
    txtCipherText: TMemo;
    btnDecrypt: TButton;
    txtPubKey: TMemo;
    txtDecipherText: TMemo;
    TabSheet3: TTabSheet;
    TabSheet6: TTabSheet;
    PageControl3: TPageControl;
    TabSheet7: TTabSheet;
    TabSheet8: TTabSheet;
    listIsaac: TListBox;
    Button5: TButton;
    Button6: TButton;
    txtRegEntropy: TMemo;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    txtNoiseSpungeKey: TMemo;
    btnMD5: TRadioButton;
    btnSHA1: TRadioButton;
    txtCryptGenRandom: TListBox;
    procedure btnHashClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnKeyGenClick(Sender: TObject);
    procedure btnEncryptClick(Sender: TObject);
    procedure btnDecryptClick(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure txtNoiseSpungeKeyKeyPress(Sender: TObject; var Key: Char);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
  private
    { Private declarations }
    fProv : TCryptoProvider;
    fKey: TCryptoKey;
    fNoiseSpunge: TKeyGenerator;
    fIsaac: TIsaac;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses CryptoAPIRNG, EncdDecd;

procedure TForm1.btnDecryptClick(Sender: TObject);
begin
  txtDecipherText.Text := fKey.Decrypt( txtCipherText.Text );
end;

procedure TForm1.btnEncryptClick(Sender: TObject);
var
  lKey: TCryptoKey;
begin
  lKey := TCryptoKey.Create(fProv, cktSignature);
  try
    lKey.PublicKey := txtPubKey.Text;
    txtCipherText.Text := lKey.Encrypt( txtPlainText.Text );
  finally
    lKey.Free;
  end;
end;

procedure TForm1.btnKeyGenClick(Sender: TObject);
var
  lCycle: string;
begin
  fKey.GenerateNewKeys;
  txtPubKey.Text := fKey.PublicKey;
  //fKey.PublicKey := txtPubKey.Text;
  lCycle := fKey.PublicKey;
  if txtPubKey.Text <> lCycle then
    raise Exception.Create('Key did not survive get/set/get cycle.');
end;

procedure TForm1.btnHashClick(Sender: TObject);
var
  lHash : TCryptoHash;
begin
  if btnSHA1.Checked then
    lHash := TCryptoHash.create(fProv, chtSHA)
  else
    lHash := TCryptoHash.create(fProv, chtMD5);
  try
    edit1.Text := lHash.HexString(memo1.text)
  finally
    lHash.Free;
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  txtRegEntropy.Text := GetCryptoSeedString;
end;

procedure TForm1.Button3Click(Sender: TObject);
var
  buf: Key256;
begin
  if not fNoiseSpunge.SafeGetKey(buf) then
    ShowMessage('Key pool empty.  Need more entropy / time.')
  else
    txtNoiseSpungeKey.Text := TCryptoWrapper.EncodeBuffer( buf, SizeOf(Buf) );
end;

procedure TForm1.Button4Click(Sender: TObject);
var
  buf: Key256;
begin
  txtNoiseSpungeKey.Text := TCryptoWrapper.EncodeBuffer( buf, SizeOf(Buf) );
end;

procedure TForm1.Button5Click(Sender: TObject);
var
  lMem: TMemoryStream;
  lStr: TStringStream;
  I: Integer;
  lIsaac: Cardinal;
begin
  lMem := TMemoryStream.Create();
  try
    for I := 1 to 8 do // 8 * 4 = 32 bytes
    begin
      lIsaac := fIsaac.val;
      lMem.Write( lIsaac, 4 );
    end;
    lMem.Position := 0;
    lStr := TStringStream.Create( '' );
    try
      EncodeStream( lMem, lStr );
      listIsaac.Items.Add( lStr.DataString );
    finally
      lStr.Free;
    end;
  finally
    lMem.Free;
  end;
end;

procedure TForm1.Button6Click(Sender: TObject);
begin
  txtCryptGenRandom.Items.Add( fProv.RandomData( 32 ) );
end;

procedure SeedIsaacFromCryptoAPI(const aIsaac: TIsaac);
var
  lSeed: array[0..19] of Cardinal;
  lSeedBuff: TSeedBuff;
  lByteAccum: Cardinal;
  I: Integer;
begin
  GetCryptoSeed(lSeedBuff);
  lByteAccum := 0;
  FillChar(lSeed, SizeOf(lSeed), 0);
  for I := 0 to C_Crypto_Seed_Length - 1 do
  begin
    lByteAccum := lByteAccum or lSeedBuff[i];
    asm
      shl lByteAccum, 2;
    end;

    if (i + 1) mod 4 = 0 then
    begin
      lSeed[i div 4] := lByteAccum;
      lByteAccum := 0;
    end;
  end;

  aIsaac.reSeed(lSeed);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  fProv := TCryptoProvider.Create( cptRSAFull, ccVerifyMachKeySet);
  fKey := TCryptoKey.Create(fProv, cktSignature);
  fKey.GenerateNewKeys;
  fNoiseSpunge := TKeyGenerator.Create;
  fIsaac := TIsaac.Create();
  SeedIsaacFromCryptoAPI(fIsaac);
  btnHash.Click;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  fKey.Free;
  fProv.Free;
  fNoiseSpunge.Free;
  fIsaac.Free;
end;

procedure TForm1.txtNoiseSpungeKeyKeyPress(Sender: TObject; var Key: Char);
begin
  fNoiseSpunge.AddLatency;
end;

end.
