// Sources:
// "NTE error codes" from http://msdn2.microsoft.com/en-us/library/aa915314.aspx
// "CRYPT Error Codes" from http://msdn2.microsoft.com/en-us/library/aa451026.aspx
unit Wcrypt2Errors;

interface

uses Windows, SysUtils;

function GetCryptErrorMessage(const aErrorValue: DWord):String;
function GetCryptErrorDescription(const aErrorValue: DWord):String;

const
  // NTE error codes
  NTE_BAD_UID                         = $80090001;
  NTE_BAD_HASH                        = $80090002;
  NTE_BAD_KEY                         = $80090003;
  NTE_BAD_LEN                         = $80090004;
  NTE_BAD_DATA                        = $80090005;
  NTE_BAD_SIGNATURE                   = $80090006;
  NTE_BAD_VER                         = $80090007;
  NTE_BAD_ALGID                       = $80090008;
  NTE_BAD_FLAGS                       = $80090009;
  NTE_BAD_TYPE                        = $8009000A;
  NTE_BAD_KEY_STATE                   = $8009000B;
  NTE_BAD_HASH_STATE                  = $8009000C;
  NTE_NO_KEY                          = $8009000D;
  NTE_NO_MEMORY                       = $8009000E;
  NTE_EXISTS                          = $8009000F;
  NTE_PERM                            = $80090010;
  NTE_NOT_FOUND                       = $80090011;
  NTE_DOUBLE_ENCRYPT                  = $80090012;
  NTE_BAD_PROVIDER                    = $80090013;
  NTE_BAD_PROV_TYPE                   = $80090014;
  NTE_BAD_PUBLIC_KEY                  = $80090015;
  NTE_BAD_KEYSET                      = $80090016;
  NTE_PROV_TYPE_NOT_DEF               = $80090017;
  NTE_PROV_TYPE_ENTRY_BAD             = $80090018;
  NTE_KEYSET_NOT_DEF                  = $80090019;
  NTE_KEYSET_ENTRY_BAD                = $8009001A;
  NTE_PROV_TYPE_NO_MATCH              = $8009001B;
  NTE_SIGNATURE_FILE_BAD              = $8009001C;
  NTE_PROVIDER_DLL_FAIL               = $8009001D;
  NTE_PROV_DLL_NOT_FOUND              = $8009001E;
  NTE_BAD_KEYSET_PARAM                = $8009001F;
  NTE_FAIL                            = $80090020;
  NTE_SYS_ERR                         = $80090021;

  // CRYPT Error Codes
  CRYPT_E_MSG_ERROR                   = $80091001;
  CRYPT_E_UNKNOWN_ALGO                = $80091002;
  CRYPT_E_OID_FORMAT                  = $80091003;
  CRYPT_E_INVALID_MSG_TYPE            = $80091004;
  CRYPT_E_UNEXPECTED_ENCODING         = $80091005;
  CRYPT_E_AUTH_ATTR_MISSING           = $80091006;
  CRYPT_E_HASH_VALUE                  = $80091007;
  CRYPT_E_INVALID_INDEX               = $80091008;
  CRYPT_E_ALREADY_DECRYPTED           = $80091009;
  CRYPT_E_NOT_DECRYPTED               = $8009100A;
  CRYPT_E_RECIPIENT_NOT_FOUND         = $8009100B;
  CRYPT_E_CONTROL_TYPE                = $8009100C;
  CRYPT_E_ISSUER_SERIALNUMBER         = $8009100D;
  CRYPT_E_SIGNER_NOT_FOUND            = $8009100E;
  CRYPT_E_ATTRIBUTES_MISSING          = $8009100F;
  CRYPT_E_STREAM_MSG_NOT_READY        = $80091010;
  CRYPT_E_STREAM_INSUFFICIENT_DATA    = $80091011;
  CRYPT_E_BAD_LEN                     = $80092001;
  CRYPT_E_BAD_ENCODE                  = $80092002;
  CRYPT_E_FILE_ERROR                  = $80092003;
  CRYPT_E_NOT_FOUND                   = $80092004;
  CRYPT_E_EXISTS                      = $80092005;
  CRYPT_E_NO_PROVIDER                 = $80092006;
  CRYPT_E_SELF_SIGNED                 = $80092007;
  CRYPT_E_DELETED_PREV                = $80092008;
  CRYPT_E_NO_MATCH                    = $80092009;
  CRYPT_E_UNEXPECTED_MSG_TYPE         = $8009200A;
  CRYPT_E_NO_KEY_PROPERTY             = $8009200B;
  CRYPT_E_NO_DECRYPT_CERT             = $8009200C;
  CRYPT_E_BAD_MSG                     = $8009200D;
  CRYPT_E_NO_SIGNER                   = $8009200E;
  CRYPT_E_PENDING_CLOSE               = $8009200F;
  CRYPT_E_REVOKED                     = $80092010;
  CRYPT_E_NO_REVOCATION_DLL           = $80092011;
  CRYPT_E_NO_REVOCATION_CHECK         = $80092012;
  CRYPT_E_REVOCATION_OFFLINE          = $80092013;
  CRYPT_E_NOT_IN_REVOCATION_DATABASE  = $80092014;
  CRYPT_E_INVALID_NUMERIC_STRING      = $80092020;
  CRYPT_E_INVALID_PRINTABLE_STRING    = $80092021;
  CRYPT_E_INVALID_IA5_STRING          = $80092022;
  CRYPT_E_INVALID_X500_STRING         = $80092023;
  CRYPT_E_NOT_CHAR_STRING             = $80092024;
  CRYPT_E_FILERESIZED                 = $80092025;
  CRYPT_E_SECURITY_SETTINGS           = $80092026;
  CRYPT_E_NO_VERIFY_USAGE_DLL         = $80092027;
  CRYPT_E_NO_VERIFY_USAGE_CHECK       = $80092028;
  CRYPT_E_VERIFY_USAGE_OFFLINE        = $80092029;
  CRYPT_E_NOT_IN_CTL                  = $8009202A;
  CRYPT_E_NO_TRUSTED_SIGNER           = $8009202B;
  CRYPT_E_OSS_ERROR                   = $80093000;

implementation

uses SysConst;

function GetCryptErrorMessage(const aErrorValue: DWord):String;
begin
  case aErrorValue of
    // NTE error messages
    NTE_BAD_UID                       : Result:= 'NTE_BAD_UID';
    NTE_BAD_HASH                      : Result:= 'NTE_BAD_HASH';
    NTE_BAD_KEY                       : Result:= 'NTE_BAD_KEY';
    NTE_BAD_LEN                       : Result:= 'NTE_BAD_LEN';
    NTE_BAD_DATA                      : Result:= 'NTE_BAD_DATA';
    NTE_BAD_SIGNATURE                 : Result:= 'NTE_BAD_SIGNATURE';
    NTE_BAD_VER                       : Result:= 'NTE_BAD_VER';
    NTE_BAD_ALGID                     : Result:= 'NTE_BAD_ALGID';
    NTE_BAD_FLAGS                     : Result:= 'NTE_BAD_FLAGS';
    NTE_BAD_TYPE                      : Result:= 'NTE_BAD_TYPE';
    NTE_BAD_KEY_STATE                 : Result:= 'NTE_BAD_KEY_STATE';
    NTE_BAD_HASH_STATE                : Result:= 'NTE_BAD_HASH_STATE';
    NTE_NO_KEY                        : Result:= 'NTE_NO_KEY';
    NTE_NO_MEMORY                     : Result:= 'NTE_NO_MEMORY';
    NTE_EXISTS                        : Result:= 'NTE_EXISTS';
    NTE_PERM                          : Result:= 'NTE_PERM';
    NTE_NOT_FOUND                     : Result:= 'NTE_NOT_FOUND';
    NTE_DOUBLE_ENCRYPT                : Result:= 'NTE_DOUBLE_ENCRYPT';
    NTE_BAD_PROVIDER                  : Result:= 'NTE_BAD_PROVIDER';
    NTE_BAD_PROV_TYPE                 : Result:= 'NTE_BAD_PROV_TYPE';
    NTE_BAD_PUBLIC_KEY                : Result:= 'NTE_BAD_PUBLIC_KEY';
    NTE_BAD_KEYSET                    : Result:= 'NTE_BAD_KEYSET';
    NTE_PROV_TYPE_NOT_DEF             : Result:= 'NTE_PROV_TYPE_NOT_DEF';
    NTE_PROV_TYPE_ENTRY_BAD           : Result:= 'NTE_PROV_TYPE_ENTRY_BAD';
    NTE_KEYSET_NOT_DEF                : Result:= 'NTE_KEYSET_NOT_DEF';
    NTE_KEYSET_ENTRY_BAD              : Result:= 'NTE_KEYSET_ENTRY_BAD';
    NTE_PROV_TYPE_NO_MATCH            : Result:= 'NTE_PROV_TYPE_NO_MATCH';
    NTE_SIGNATURE_FILE_BAD            : Result:= 'NTE_SIGNATURE_FILE_BAD';
    NTE_PROVIDER_DLL_FAIL             : Result:= 'NTE_PROVIDER_DLL_FAIL';
    NTE_PROV_DLL_NOT_FOUND            : Result:= 'NTE_PROV_DLL_NOT_FOUND';
    NTE_BAD_KEYSET_PARAM              : Result:= 'NTE_BAD_KEYSET_PARAM';
    NTE_FAIL                          : Result:= 'NTE_FAIL';
    NTE_SYS_ERR                       : Result:= 'NTE_SYS_ERR';

    // CRYPT error messages
    CRYPT_E_MSG_ERROR                 : Result := 'CRYPT_E_MSG_ERROR';
    CRYPT_E_UNKNOWN_ALGO              : Result := 'CRYPT_E_UNKNOWN_ALGO';
    CRYPT_E_OID_FORMAT                : Result := 'CRYPT_E_OID_FORMAT';
    CRYPT_E_INVALID_MSG_TYPE          : Result := 'CRYPT_E_INVALID_MSG_TYPE';
    CRYPT_E_UNEXPECTED_ENCODING       : Result := 'CRYPT_E_UNEXPECTED_ENCODING';
    CRYPT_E_AUTH_ATTR_MISSING         : Result := 'CRYPT_E_AUTH_ATTR_MISSING';
    CRYPT_E_HASH_VALUE                : Result := 'CRYPT_E_HASH_VALUE';
    CRYPT_E_INVALID_INDEX             : Result := 'CRYPT_E_INVALID_INDEX';
    CRYPT_E_ALREADY_DECRYPTED         : Result := 'CRYPT_E_ALREADY_DECRYPTED';
    CRYPT_E_NOT_DECRYPTED             : Result := 'CRYPT_E_NOT_DECRYPTED';
    CRYPT_E_RECIPIENT_NOT_FOUND       : Result := 'CRYPT_E_RECIPIENT_NOT_FOUND';
    CRYPT_E_CONTROL_TYPE              : Result := 'CRYPT_E_CONTROL_TYPE';
    CRYPT_E_ISSUER_SERIALNUMBER       : Result := 'CRYPT_E_ISSUER_SERIALNUMBER';
    CRYPT_E_SIGNER_NOT_FOUND          : Result := 'CRYPT_E_SIGNER_NOT_FOUND';
    CRYPT_E_ATTRIBUTES_MISSING        : Result := 'CRYPT_E_ATTRIBUTES_MISSING';
    CRYPT_E_STREAM_MSG_NOT_READY      : Result := 'CRYPT_E_STREAM_MSG_NOT_READY';
    CRYPT_E_STREAM_INSUFFICIENT_DATA  : Result := 'CRYPT_E_STREAM_INSUFFICIENT_DATA';
    CRYPT_E_BAD_LEN                   : Result := 'CRYPT_E_BAD_LEN';
    CRYPT_E_BAD_ENCODE                : Result := 'CRYPT_E_BAD_ENCODE';
    CRYPT_E_FILE_ERROR                : Result := 'CRYPT_E_FILE_ERROR';
    CRYPT_E_NOT_FOUND                 : Result := 'CRYPT_E_NOT_FOUND';
    CRYPT_E_EXISTS                    : Result := 'CRYPT_E_EXISTS';
    CRYPT_E_NO_PROVIDER               : Result := 'CRYPT_E_NO_PROVIDER';
    CRYPT_E_SELF_SIGNED               : Result := 'CRYPT_E_SELF_SIGNED';
    CRYPT_E_DELETED_PREV              : Result := 'CRYPT_E_DELETED_PREV';
    CRYPT_E_NO_MATCH                  : Result := 'CRYPT_E_NO_MATCH';
    CRYPT_E_UNEXPECTED_MSG_TYPE       : Result := 'CRYPT_E_UNEXPECTED_MSG_TYPE';
    CRYPT_E_NO_KEY_PROPERTY           : Result := 'CRYPT_E_NO_KEY_PROPERTY';
    CRYPT_E_NO_DECRYPT_CERT           : Result := 'CRYPT_E_NO_DECRYPT_CERT';
    CRYPT_E_BAD_MSG                   : Result := 'CRYPT_E_BAD_MSG';
    CRYPT_E_NO_SIGNER                 : Result := 'CRYPT_E_NO_SIGNER';
    CRYPT_E_PENDING_CLOSE             : Result := 'CRYPT_E_PENDING_CLOSE';
    CRYPT_E_REVOKED                   : Result := 'CRYPT_E_REVOKED';
    CRYPT_E_NO_REVOCATION_DLL         : Result := 'CRYPT_E_NO_REVOCATION_DLL';
    CRYPT_E_NO_REVOCATION_CHECK       : Result := 'CRYPT_E_NO_REVOCATION_CHECK';
    CRYPT_E_REVOCATION_OFFLINE        : Result := 'CRYPT_E_REVOCATION_OFFLINE';
    CRYPT_E_NOT_IN_REVOCATION_DATABASE: Result := 'CRYPT_E_NOT_IN_REVOCATION_DATABASE';
    CRYPT_E_INVALID_NUMERIC_STRING    : Result := 'CRYPT_E_INVALID_NUMERIC_STRING';
    CRYPT_E_INVALID_PRINTABLE_STRING  : Result := 'CRYPT_E_INVALID_PRINTABLE_STRING';
    CRYPT_E_INVALID_IA5_STRING        : Result := 'CRYPT_E_INVALID_IA5_STRING';
    CRYPT_E_INVALID_X500_STRING       : Result := 'CRYPT_E_INVALID_X500_STRING';
    CRYPT_E_NOT_CHAR_STRING           : Result := 'CRYPT_E_NOT_CHAR_STRING';
    CRYPT_E_FILERESIZED               : Result := 'CRYPT_E_FILERESIZED';
    CRYPT_E_SECURITY_SETTINGS         : Result := 'CRYPT_E_SECURITY_SETTINGS';
    CRYPT_E_NO_VERIFY_USAGE_DLL       : Result := 'CRYPT_E_NO_VERIFY_USAGE_DLL';
    CRYPT_E_NO_VERIFY_USAGE_CHECK     : Result := 'CRYPT_E_NO_VERIFY_USAGE_CHECK';
    CRYPT_E_VERIFY_USAGE_OFFLINE      : Result := 'CRYPT_E_VERIFY_USAGE_OFFLINE';
    CRYPT_E_NOT_IN_CTL                : Result := 'CRYPT_E_NOT_IN_CTL';
    CRYPT_E_NO_TRUSTED_SIGNER         : Result := 'CRYPT_E_NO_TRUSTED_SIGNER';
    CRYPT_E_OSS_ERROR                 : Result := 'CRYPT_E_OSS_ERROR';
  else
    Result := Format(SOSError, [aErrorValue, SysErrorMessage(aErrorValue)]);
  end;
end;

function GetCryptErrorDescription(const aErrorValue: DWord):String;
begin
  case aErrorValue of
     // NTE error descriptions

    NTE_BAD_UID                       : Result := 'Bad UID.';
    NTE_BAD_HASH                      : Result := 'Bad hash.';
    NTE_BAD_KEY                       : Result := 'Bad key.';
    NTE_BAD_LEN                       : Result := 'Bad length.';
    NTE_BAD_DATA                      : Result := 'Bad data.';
    NTE_BAD_SIGNATURE                 : Result := 'Invalid signature.';
    NTE_BAD_VER                       : Result := 'Bad version of provider.';
    NTE_BAD_ALGID                     : Result := 'Invalid algorithm specified.';
    NTE_BAD_FLAGS                     : Result := 'Invalid flags specified.';
    NTE_BAD_TYPE                      : Result := 'Invalid type specified.';
    NTE_BAD_KEY_STATE                 : Result := 'Key not valid for use in specified state.';
    NTE_BAD_HASH_STATE                : Result := 'Hash not valid for use in specified state.';
    NTE_NO_KEY                        : Result := 'Key does not exist.';
    NTE_NO_MEMORY                     : Result := 'Insufficient memory available for the operation.';
    NTE_EXISTS                        : Result := 'Object already exists.';
    NTE_PERM                          : Result := 'Access denied.';
    NTE_NOT_FOUND                     : Result := 'Object was not found.';
    NTE_DOUBLE_ENCRYPT                : Result := 'Data already encrypted.';
    NTE_BAD_PROVIDER                  : Result := 'Invalid provider specified.';
    NTE_BAD_PROV_TYPE                 : Result := 'Invalid provider type specified.';
    NTE_BAD_PUBLIC_KEY                : Result := 'Invalid provider public key.';
    NTE_BAD_KEYSET                    : Result := 'Keyset does not exist';
    NTE_PROV_TYPE_NOT_DEF             : Result := 'Provider type not defined.';
    NTE_PROV_TYPE_ENTRY_BAD           : Result := 'Invalid registration for provider type.';
    NTE_KEYSET_NOT_DEF                : Result := 'The keyset not defined.';
    NTE_KEYSET_ENTRY_BAD              : Result := 'Invalid keyset registration.';
    NTE_PROV_TYPE_NO_MATCH            : Result := 'Provider type does not match registered value.';
    NTE_SIGNATURE_FILE_BAD            : Result := 'Corrupt digital signature file.';
    NTE_PROVIDER_DLL_FAIL             : Result := 'Provider DLL failed to initialize correctly.';
    NTE_PROV_DLL_NOT_FOUND            : Result := 'Provider DLL not found.';
    NTE_BAD_KEYSET_PARAM              : Result := 'Invalid keyset parameter.';
    NTE_FAIL                          : Result := 'Internal error occurred.';
    NTE_SYS_ERR                       : Result := 'Base error occurred.';

    // CRYPT error descriptions
    CRYPT_E_MSG_ERROR                 : Result := 'An error was encountered doing a cryptographic message operation.';
    CRYPT_E_UNKNOWN_ALGO              : Result := 'The cryptographic algorithm is unknown.';
    CRYPT_E_OID_FORMAT                : Result := 'The object identifier is badly formatted.';
    CRYPT_E_INVALID_MSG_TYPE          : Result := 'The message type is invalid.';
    CRYPT_E_UNEXPECTED_ENCODING       : Result := 'Unexpected message encoding.';
    CRYPT_E_AUTH_ATTR_MISSING         : Result := 'Missing an expected authenticated attribute.';
    CRYPT_E_HASH_VALUE                : Result := 'Incorrect hash value.';
    CRYPT_E_INVALID_INDEX             : Result := 'Invalid index value.';
    CRYPT_E_ALREADY_DECRYPTED         : Result := 'Message content already decrypted.';
    CRYPT_E_NOT_DECRYPTED             : Result := 'Message content not yet decrypted.';
    CRYPT_E_RECIPIENT_NOT_FOUND       : Result := 'Enveloped-data message missing the specified recipient.';
    CRYPT_E_CONTROL_TYPE              : Result := 'Invalid control type.';
    CRYPT_E_ISSUER_SERIALNUMBER       : Result := 'Invalid issuer or serial number.';
    CRYPT_E_SIGNER_NOT_FOUND          : Result := 'Original signer not found.';
    CRYPT_E_ATTRIBUTES_MISSING        : Result := 'Message missing requested attributes.';
    CRYPT_E_STREAM_MSG_NOT_READY      : Result := 'Streamed message is note yet able to return the requested data.';
    CRYPT_E_STREAM_INSUFFICIENT_DATA  : Result := 'Streamed message needs more data.';
    CRYPT_E_BAD_LEN                   : Result := 'Insufficient output data length specified.';
    CRYPT_E_BAD_ENCODE                : Result := 'Error while encoding or decoding.';
    CRYPT_E_FILE_ERROR                : Result := 'Error while reading or writing to the file';
    CRYPT_E_NOT_FOUND                 : Result := 'Object or property not found.';
    CRYPT_E_EXISTS                    : Result := 'Object or property already exists.';
    CRYPT_E_NO_PROVIDER               : Result := 'No provider specified for store or object.';
    CRYPT_E_SELF_SIGNED               : Result := 'Specified certificate is self-signed.';
    CRYPT_E_DELETED_PREV              : Result := 'Previous certificate or CRL context deleted.';
    CRYPT_E_NO_MATCH                  : Result := 'No match when trying to find the object.';
    CRYPT_E_UNEXPECTED_MSG_TYPE       : Result := 'The type of the cryptographic message being decoded is different than what was expected.';
    CRYPT_E_NO_KEY_PROPERTY           : Result := 'The certificate doesn''t have a private key property';
    CRYPT_E_NO_DECRYPT_CERT           : Result := 'No certificate was found having a private key property to use for decrypting.';
    CRYPT_E_BAD_MSG                   : Result := 'Not a cryptographic message, or incorrectly formatted.';
    CRYPT_E_NO_SIGNER                 : Result := 'The signed message doesn''t have a signer for the specified signer index';
    CRYPT_E_PENDING_CLOSE             : Result := 'Final closure is pending until additional frees or closes.';
    CRYPT_E_REVOKED                   : Result := 'The certificate or signature has been revoked';
    CRYPT_E_NO_REVOCATION_DLL         : Result := 'No Dll or exported function found to verify revocation.';
    CRYPT_E_NO_REVOCATION_CHECK       : Result := 'Called function unable to perform revocation check.';
    CRYPT_E_REVOCATION_OFFLINE        : Result := 'Because the revocation server was offline, the called function wasn''t able to complete the revocation check.';
    CRYPT_E_NOT_IN_REVOCATION_DATABASE: Result := 'The certificate or signature not found in the revocation server database.';
    CRYPT_E_INVALID_NUMERIC_STRING    : Result := 'The string contains a non-numeric character.';
    CRYPT_E_INVALID_PRINTABLE_STRING  : Result := 'The string contains a non-printable character.';
    CRYPT_E_INVALID_IA5_STRING        : Result := 'The string contains a character not in the 7 bit ASCII character set.';
    CRYPT_E_INVALID_X500_STRING       : Result := 'The string contains an invalid X500 name attribute key, OID, value or delimiter.';
    CRYPT_E_NOT_CHAR_STRING           : Result := 'The dwvaluetype for CERT_NAME_VALUE not one of the character strings.';
    CRYPT_E_FILERESIZED               : Result := 'Put operation cannot continue. The file needs to be resized.';
    CRYPT_E_SECURITY_SETTINGS         : Result := 'The cryptography operation failed due to a local security option setting.';
    CRYPT_E_NO_VERIFY_USAGE_DLL       : Result := 'No DLL or exported function found to verify subject usage.';
    CRYPT_E_NO_VERIFY_USAGE_CHECK     : Result := 'The called function was not able to do a usage check on the subject.';
    CRYPT_E_VERIFY_USAGE_OFFLINE      : Result := 'Because the server was offline, the called function was not able to complete the usage check.';
    CRYPT_E_NOT_IN_CTL                : Result := 'The subject not found in a Certificate Trust List.';
    CRYPT_E_NO_TRUSTED_SIGNER         : Result := 'No trusted signer found.';
    CRYPT_E_OSS_ERROR                 : Result := 'OSS Certificate encode/decode error code base. See asn1code.h.';
  else
    Result := Format(SOSError, [aErrorValue, SysErrorMessage(aErrorValue)]);
  end;
end;

end.
